@@ -44,7 +44,7 @@
 #include <help.xpm>
 #include <icon.xpm>
 
-#define VERSION_STRING "0.8.4"
+#define VERSION_STRING "0.8.8"
 
 #include <iostream>
 
@@ -93,7 +93,12 @@ MainWin::MainWin( QWidget *parent, const char *name ) :
            this, SLOT( setUsesTextLabel( bool ) ));
   connect( m_wid, SIGNAL( setConnect( bool ) ),
            this, SLOT( setConnectSLOT( bool ) ));
-
+  connect( m_wid, SIGNAL( toolbarVisibility( bool, bool, bool, bool, bool )),
+           this, SLOT( toolbarVisibilitySLOT( bool, bool, bool, bool, bool ) ));
+  
+  connect( m_wid, SIGNAL( connectDMM( bool ) ),
+           m_connectAction, SLOT( setOn( bool ) ));
+  
   QRect winRect = m_wid->winRect();
   m_wid->applySLOT();  
     
@@ -124,13 +129,12 @@ MainWin::MainWin( QWidget *parent, const char *name ) :
 MainWin::~MainWin()
 {
 }
-
+ 
 void
 MainWin::setConsoleLogging( bool on )
 {
   m_wid->setConsoleLogging( on );
 }
-
       
 void
 MainWin::createActions()
@@ -267,6 +271,8 @@ MainWin::createActions()
            m_wid, SLOT( configDmmSLOT() ));
   connect( m_configRecorderAction, SIGNAL( activated() ),
            m_wid, SLOT( configRecorderSLOT() ));
+  connect( m_quitAction, SIGNAL( activated() ),
+           this, SLOT( setToolbarVisibilitySLOT() ));
   connect( m_quitAction, SIGNAL( activated() ),
            m_wid, SLOT( quitSLOT() ));
   connect( m_helpAction, SIGNAL( activated() ),
@@ -314,15 +320,13 @@ MainWin::versionSLOT()
   msg += ver;
   msg += "</h1><hr>"
          "<div align=right><i>A simple recorder for DMM's</i></div><p>"
-         "<div align=justify>A simple display software for a variety of digital multimeter such as:<p>"
-         "<table><tr><td><b>ELV</b></td><td>M9803R</td></tr>"
-         "<tr><td><b>Metex</b></td><td>M-3660D, M-3830D, M-3850D, ME-11, ME-22, ME-32 and Universal system 9160</td></tr>"
-         "<tr><td><b>PeakTech</b></td><td>4010 and 451</td></tr>"
-         "<tr><td><b>RadioShack</b></td><td>22-805</td></tr>"
-         "<tr><td><b>Voltcraft</b></td><td>M-3650D, M-4660, ME-11, ME-22T, ME-32 and VC&nbsp;670</td></tr>"
-         "<tr><td colspan=2>Implemented, but not yet confirmed by a user are:</td></tr>"
-         "<tr><td><b>Voltcraft</b></td><td>ME-42, M-3860, M-4660A, M-4660M, MXD-4660A, VC&nbsp;630, VC&nbsp;650"
-         ", VC&nbsp;635 and VC&nbsp;655</td></tr></table>"
+         "<div align=justify>A simple display software for a variety of digital multimeter. Currently confirmed are:";
+
+  msg += "<table>";
+  msg += m_wid->deviceListText();
+  msg += "</table>";
+
+  msg +=
          "Other compatible models may work also.<p>"
          "QtDMM features min/max memory and a configurable "
          "recorder with import/export and printing function. Sampling may"
@@ -333,8 +337,8 @@ MainWin::versionSLOT()
   msg += qVersion();
   msg += " from Trolltech AS Norway <font color=blue><u>www.trolltech.com</u></font>"
          " and is licensed under <b>GPL</b>.</div><br>"
-         "&copy; 2001-2005 Matthias Toussaint &nbsp;-&nbsp;&nbsp;<font color=blue><u>qtdmm@mtoussaint.de</u></font>"
-         "<p><br>The icons (except the DMM icon) are taken from the KDE project.<p>";
+         "&copy; 2001-2006 Matthias Toussaint &nbsp;-&nbsp;&nbsp;<font color=blue><u>qtdmm@mtoussaint.de</u></font>"
+         "<p><br>The icons (except the DMM icon) have been taken from the KDE project.<p>";
           
   QMessageBox version( tr("QtDMM: Welcome!" ),
                        tr( msg ),
@@ -347,7 +351,7 @@ MainWin::versionSLOT()
   version.setIconPixmap( QPixmap((const char **)icon_xpm ) );
   version.exec();
 }
-    
+
 void
 MainWin::createToolBars()
 {
@@ -375,10 +379,22 @@ MainWin::createToolBars()
   
   m_helpTB = new QToolBar( this );
   m_helpAction->addTo( m_helpTB );
+  addToolBar( m_helpTB, tr("Help") );
   
   m_displayTB = new QToolBar( this );
   m_display = new DisplayWid( m_displayTB );
   addToolBar( m_displayTB, tr("Display"), Top, true );
+  
+  connect( m_displayTB, SIGNAL( visibilityChanged( bool ) ),
+           this, SLOT( setToolbarVisibilitySLOT() ));
+  connect( m_helpTB, SIGNAL( visibilityChanged( bool ) ),
+           this, SLOT( setToolbarVisibilitySLOT() ));
+  connect( m_fileTB, SIGNAL( visibilityChanged( bool ) ),
+           this, SLOT( setToolbarVisibilitySLOT() ));
+  connect( m_graphTB, SIGNAL( visibilityChanged( bool ) ),
+           this, SLOT( setToolbarVisibilitySLOT() ));
+  connect( m_dmmTB, SIGNAL( visibilityChanged( bool ) ),
+           this, SLOT( setToolbarVisibilitySLOT() ));
 }
 
 void
@@ -429,6 +445,8 @@ MainWin::createMenu()
 void
 MainWin::closeEvent( QCloseEvent *ev )
 {
+  setToolbarVisibilitySLOT();
+                               
   if (m_wid->closeWin())
   {
     ev->accept();
@@ -439,8 +457,29 @@ MainWin::closeEvent( QCloseEvent *ev )
   } 
 }
 
+void MainWin::setToolbarVisibilitySLOT()
+{
+  m_wid->setToolbarVisibility( m_displayTB->isVisible(),
+                               m_dmmTB->isVisible(),
+                               m_graphTB->isVisible(),
+                               m_fileTB->isVisible(),
+                               m_helpTB->isVisible() );
+}
+
 void
 MainWin::setConnectSLOT( bool on )
 {
   m_connectAction->setOn( on );
 }
+
+void
+MainWin::toolbarVisibilitySLOT( bool disp, bool dmm, bool graph, 
+                                bool file, bool help )
+{
+  m_dmmTB->setShown( dmm );
+  m_graphTB->setShown( graph );
+  m_fileTB->setShown( file );
+  m_helpTB->setShown( help );
+  m_displayTB->setShown( disp );
+}
+
